@@ -857,9 +857,7 @@ SV *p5_call_gv(PerlInterpreter *my_perl, GV *gv, int len, SV *args[], I32 *count
 
         handle_p5_error(err);
         retval = pop_return_values(my_perl, sp, *count, type);
-        SPAGAIN;
 
-        PUTBACK;
         FREETMPS;
         LEAVE;
 
@@ -916,12 +914,11 @@ SV *p5_call_parent_gv(PerlInterpreter *my_perl, GV *gv, int len, SV *args[], I32
 
         handle_p5_error(err);
         retval = pop_return_values(my_perl, sp, *count, type);
-        SPAGAIN;
 
         reset_wrapped_object(my_perl, obj);
+        SPAGAIN;
         SvREFCNT_dec(obj);
 
-        PUTBACK;
         FREETMPS;
         LEAVE;
 
@@ -961,7 +958,6 @@ SV *p5_scalar_call_gv(PerlInterpreter *my_perl, GV *gv, int len, SV *args[], I32
 
         handle_p5_error(err);
         retval = pop_return_values(my_perl, sp, *count, type);
-        SPAGAIN;
 
         PUTBACK;
         FREETMPS;
@@ -1003,9 +999,7 @@ SV *p5_scalar_call_parent_gv(PerlInterpreter *my_perl, GV *gv, int len, SV *args
 
         handle_p5_error(err);
         retval = pop_return_values(my_perl, sp, *count, type);
-        SPAGAIN;
 
-        PUTBACK;
         FREETMPS;
         LEAVE;
 
@@ -1036,14 +1030,11 @@ SV *p5_call_gv_two_args(PerlInterpreter *my_perl, GV *gv, SV *arg, SV *arg2, I32
         SPAGAIN;
 
         handle_p5_error(err);
-        if (*err)
-            fprintf(stderr, "err: %d\n", *err);
         retval = pop_return_values(my_perl, sp, *count, type);
-        SPAGAIN;
 
         reset_wrapped_object(my_perl, arg);
+        SPAGAIN;
 
-        PUTBACK;
         FREETMPS;
         LEAVE;
 
@@ -1074,12 +1065,8 @@ SV *p5_scalar_call_gv_two_args(PerlInterpreter *my_perl, GV *gv, SV *arg, SV *ar
         SPAGAIN;
 
         handle_p5_error(err);
-        if (*err)
-            fprintf(stderr, "err: %d\n", *err);
         retval = pop_return_values(my_perl, sp, *count, type);
-        SPAGAIN;
 
-        PUTBACK;
         FREETMPS;
         LEAVE;
 
@@ -1123,13 +1110,11 @@ SV *p5_call_method(PerlInterpreter *my_perl, SV *obj, I32 context, char *name, i
 
             handle_p5_error(err);
             retval = pop_return_values(my_perl, sp, *count, type);
-            SPAGAIN;
         }
         else {
             ERRSV = newSVpvf("Could not find method \"%s\" of \"%s\" object", name, HvNAME(pkg));
         }
 
-        PUTBACK;
         FREETMPS;
         LEAVE;
 
@@ -1177,15 +1162,14 @@ SV *p5_call_parent_method(PerlInterpreter *my_perl, char *package, SV *parent_ob
 
             handle_p5_error(err);
             retval = pop_return_values(my_perl, sp, *count, type);
-            SPAGAIN;
 
             if (p5_is_live_wrapped_p6_object(my_perl, SvRV(obj))) {
                 reset_wrapped_object(my_perl, obj);
+                SPAGAIN;
                 SvREFCNT_dec(obj);
             }
         }
 
-        PUTBACK;
         FREETMPS;
         LEAVE;
 
@@ -1582,7 +1566,7 @@ void post_callback(const I32 ax, SV **sp, I32 items, SV * const args_rv, SV *err
     handle_p6_error(err);
     sv_2mortal(retval);
     sp -= items;
-    return return_retval(ax, sp, retval);
+    return_retval(ax, sp, retval);
 }
 
 XS(p5_call_p6_method) {
@@ -1619,7 +1603,7 @@ XS(p5_call_p6_method) {
 
     declare_cbs;
     SV * retval = cbs->call_p6_method(p6mg->index, name_pv, GIMME_V == G_SCALAR, args_rv, &err);
-    return post_callback(ax, sp, items, args_rv, err, retval);
+    post_callback(ax, sp, items, args_rv, err, retval);
 }
 
 XS(p5_destroy_p5_object) {
@@ -1706,7 +1690,8 @@ XS(p5_call_p6_extension_method) {
             SV * const args_rv = newRV_noinc((SV *) args);
             declare_cbs;
             SV * retval = cbs->call_p6_package_method(package_pv, name_pv, GIMME_V == G_SCALAR, args_rv, &err);
-            return post_callback(ax, sp, items, args_rv, err, retval);
+            post_callback(ax, sp, items, args_rv, err, retval);
+            return;
         }
         else {
             croak("Got a non-reference for obj in p5_call_p6_extension_method?!");
@@ -1720,7 +1705,7 @@ XS(p5_call_p6_extension_method) {
 
     declare_cbs;
     SV * retval = cbs->call_p6_method(p6mg->index, name_pv, GIMME_V == G_SCALAR, args_rv, &err);
-    return post_callback(ax, sp, items, args_rv, err, retval);
+    post_callback(ax, sp, items, args_rv, err, retval);
 }
 
 XS(p5_hash_at_key) {
@@ -1777,7 +1762,7 @@ XS(p5_call_p6_callable) {
 
     declare_cbs;
     SV * retval = cbs->call_p6_callable(SvIV(index), args_rv, &err);
-    return post_callback(ax, sp, items, args_rv, err, retval);
+    post_callback(ax, sp, items, args_rv, err, retval);
 }
 
 XS(p5_load_module) {
